/**
 * Enrich generated data from signatures of source files (if found).
 * Currently enriching only method meta object and hasshes if method is async.
 * @param generatedSignature
 * @param sourceSignatures
 * @param logger
 * @constructor
 */
function SourceDataEnricher(generatedSignature, sourceSignatures, logger) {
    this.generatedSignature = generatedSignature;
    this.sourceSignatures = sourceSignatures;
    this.logger = logger;
}

/**
 * Iterate over generated and source methods, find match by source position and enrich method data.
 */
SourceDataEnricher.prototype.enrichMethods = function(){
    var context = this;
    context.generatedSignature.sourceFileToMethodMap = context.generatedSignature.sourceFileToMethodMap || {};
    this.sourceSignatures.forEach(function (sourceSig) {
        if(!sourceSig) {
            context.logger.error("Source signature is undefined, skipping enrichment")
            return;
        }
        var sourcePositionToGeneratedMethodMap = context.generatedSignature.sourceFileToMethodMap[sourceSig.absolutePath];
        Object.keys(sourcePositionToGeneratedMethodMap).forEach(function (position) {
            var sourceMethodIndex = sourceSig.positionToMethodMap[position];
            var generatedMethodIndex = sourcePositionToGeneratedMethodMap[position];
            if(sourceMethodIndex == null || generatedMethodIndex == null){
                context.logger.debug("Could find match for method at '%s' in file '%s'. " +
                 "sourceMethodIndex: '%s', generatedMethodIndex: '%s'.", position, sourceSig.absolutePath, sourceMethodIndex,
                 generatedMethodIndex)
                return;
            }
            var sourceHash = sourceSig.methods[sourceMethodIndex].generatedData.hash;
            var sourceSigHash = sourceSig.methods[sourceMethodIndex].generatedData.sigHash;
            context.generatedSignature.methods[generatedMethodIndex].meta = sourceSig.methods[sourceMethodIndex].meta;
            if(sourceSig.methods[sourceMethodIndex].meta.isAsync){
                context.generatedSignature.methods[generatedMethodIndex].srcData.hash = sourceHash;
                context.generatedSignature.methods[generatedMethodIndex].srcData.sigHash = sourceSigHash;
            }
        })

    })
}

module.exports = SourceDataEnricher;